/*
 *  Gideon, a GUI designer for GTK+
 *  Copyright (c) 2005 Maxim Udushlivy
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */

#ifndef GIDEON_GIDEON_H
#define GIDEON_GIDEON_H

#include <gtk/gtk.h>
#include <guiloader.h>

#define GIDEON_VERSION (2*1000000 + 7*1000 + 1)

#ifdef _WIN32
	#ifdef GIDEON_LIBRARY
		#define GIDEON_API __declspec(dllexport)
	#else
		#define GIDEON_API __declspec(dllimport)
	#endif
#else
	#define GIDEON_API __attribute__ ((visibility("default")))
#endif

G_BEGIN_DECLS

typedef struct _GideonDesigner GideonDesigner;

GIDEON_API gint gideon_version(void);

GIDEON_API void gideon_enter_check(const gchar * msg);
GIDEON_API void gideon_leave(void);

GIDEON_API GideonDesigner * gideon_designer_new(void);
GIDEON_API void gideon_designer_extend_ui(GideonDesigner * designer, const gchar * ui, GtkActionGroup * actions);
GIDEON_API void gideon_designer_apply_settings(GideonDesigner * designer, GHashTable * settings);
GIDEON_API void gideon_designer_store_settings(GideonDesigner * designer, GHashTable * settings);
GIDEON_API gboolean gideon_designer_can_save(GideonDesigner * designer);
GIDEON_API gboolean gideon_designer_should_save(GideonDesigner * designer);
GIDEON_API void gideon_designer_clear(GideonDesigner * designer);
GIDEON_API void gideon_designer_load(GideonDesigner * designer, const gchar * data);
GIDEON_API gchar * gideon_designer_save(GideonDesigner * designer);
GIDEON_API void gideon_designer_saved(GideonDesigner * designer);
GIDEON_API GtkAccelGroup * gideon_designer_get_accel_group(GideonDesigner * designer);
GIDEON_API gchar * gideon_designer_save_header(GideonDesigner * designer);
GIDEON_API void gideon_designer_set_embedded(GideonDesigner * designer, gboolean embedded);

#define gideon_enter() gideon_enter_check(NULL)

G_END_DECLS

#if defined(__cplusplus) && !defined(GIDEON_DISABLE_DEPRECATED)

#include <string>
#include <guiloader-c++.h>

#define GIDEON_GUI_DECL               \
  {                                   \
    const char * data=

#define GIDEON_GUI_LOAD(loader)       \
    ;                                 \
    loader->load_from_string(data);   \
  }

#define GIDEON_(SSS)

namespace Gideon {

class Loader;
class Factory;

typedef Glib::RefPtr<Loader> PLoader;
typedef Glib::RefPtr<Factory> PFactory;

class Loader : public Glib::Object {

public:
	Loader() { loader= Gui::Loader::create(); loader->set_steal_objects(true); }

	void load_from_string(const Glib::ustring & data) { loader->load_from_string(data); }
	void load_from_file(const std::string & filename) { loader->load_from_file(filename); }

	template <class S, class T> void connect_s(const std::string & name, const T & slot, int count=1)
		{ loader->connect_signals_s<S>(name, slot, count); }
	template <class S, class T> void connect(const std::string & name, const T & slot, int count=1) 
		{ loader->connect_signals<S>(name, slot, count); }
	template <class T> void connect_s(const std::string & name, const T & slot, int count=1) 
		{ loader->connect_signals_s(name, slot, count); }
	template <class T> void connect(const std::string & name, const T & slot, int count=1) 
		{ loader->connect_signals(name, slot, count); }

	template <class T> void bind(const std::string & name, Glib::RefPtr<T> * ptr)
		{ loader->bind_object(name, ptr); }
	template <class T> void bind(const std::string & name, T * * ptr)
		{ loader->bind_object(name, ptr); }

private:
	Gui::Enter enter;
	Gui::PLoader loader;

};

class Factory : public Glib::Object {

public:
	static PFactory get() { return PFactory(new Factory()); }
	PLoader create_loader() { return PLoader(new Loader()); }

private:
	Gui::Enter enter;

};

}

#endif

#endif
